@@ -43,6 +43,10 @@ class Searchbar(Gtk.HBox):
 
         self.config = Config()
 
+        if not self.config.base.get_item('case_sensitive'):
+            self.regex_flags_pcre2 |= regex.PCRE2_CASELESS
+            self.regex_flags_glib |= regex.GLIB_CASELESS
+
         self.get_style_context().add_class("terminator-terminal-searchbar")
 
         # Search text
@@ -55,6 +59,8 @@ class Searchbar(Gtk.HBox):
         # Label
         label = Gtk.Label(label=_('Search:'))
         label.show()
+        label.set_margin_start(10)
+        label.set_margin_end(5)
 
         # Close Button
         close = Gtk.Button()
@@ -85,7 +91,7 @@ class Searchbar(Gtk.HBox):
         self.match_case = Gtk.CheckButton.new_with_label('Match Case')
         self.match_case.show()
         self.match_case.set_sensitive(True)
-        self.match_case.set_active(True)
+        self.match_case.set_active(self.config.base.get_item('case_sensitive'))
         self.match_case.connect('toggled', self.match_case_toggled)
 
         # Wrap checkbox
@@ -95,17 +101,31 @@ class Searchbar(Gtk.HBox):
         self.wrap.set_active(True)
         self.wrap.connect('toggled', self.wrap_toggled)
 
+        # Invert Search checkbox
+        self.invert_search = Gtk.CheckButton.new_with_label('Invert Search')
+        self.invert_search.show()
+        self.search_is_inverted = self.config.base.get_item('invert_search')
+        self.invert_search.set_active(self.search_is_inverted)
+        self.invert_search.connect('toggled', self.wrap_invert_search)
+
         self.pack_start(label, False, True, 0)
         self.pack_start(self.entry, True, True, 0)
         self.pack_start(self.prev, False, False, 0)
         self.pack_start(self.next, False, False, 0)
         self.pack_start(self.wrap, False, False, 0)
         self.pack_start(self.match_case, False, False, 0)
+        self.pack_start(self.invert_search, False, False, 0)
         self.pack_end(close, False, False, 0)
 
         self.hide()
         self.set_no_show_all(True)
 
+    def wrap_invert_search(self, toggled):
+        self.search_is_inverted = toggled.get_active()
+        self.invert_search.set_active(toggled.get_active())
+        self.config.base.set_item('invert_search', toggled.get_active())
+        self.config.save()
+
     def wrap_toggled(self, toggled):
         toggled_state = toggled.get_active()
         self.vte.search_set_wrap_around(toggled_state)
@@ -132,7 +152,9 @@ class Searchbar(Gtk.HBox):
             self.regex_flags_pcre2 = regex.FLAGS_PCRE2
             self.regex_flags_glib = regex.FLAGS_GLIB
         
-        self.do_search(self.entry) #  Start a new search everytime the check box is toggled.    
+        self.config.base.set_item('case_sensitive', toggled_state)
+        self.config.save()
+        self.do_search(self.entry) #  Start a new search every time the check box is toggled.    
 
     def get_vte(self):
         """Find our parent widget"""
@@ -148,9 +170,13 @@ class Searchbar(Gtk.HBox):
         key = Gdk.keyval_name(event.keyval)
         if key == 'Escape':
             self.end_search()
-        else:
-            self.prev.set_sensitive(False)
-            self.next.set_sensitive(False)
+        elif (event.state & Gdk.ModifierType.SHIFT_MASK)\
+                and (event.keyval == Gdk.KEY_Return or event.keyval == Gdk.KEY_KP_Enter):
+            if self.search_is_inverted:
+                self.next_search(None)
+            else:
+                self.prev_search(None)
+            return True
 
     def start_search(self):
         """Show ourselves"""
@@ -186,7 +212,11 @@ class Searchbar(Gtk.HBox):
 
         self.next.set_sensitive(True)
         self.prev.set_sensitive(True)
-        self.next_search(None)
+        # switch search direction based on the inversion checkbox
+        if not self.search_is_inverted:
+            self.next_search(None)
+        else:
+            self.prev_search(None)
 
     def next_search(self, widget):
         """Search forwards and jump to the next result, if any"""
